public inbox for passt-dev@passt.top
 help / color / mirror / code / Atom feed
From: Laurent Vivier <lvivier@redhat.com>
To: passt-dev@passt.top
Cc: Laurent Vivier <lvivier@redhat.com>
Subject: [PATCH 01/18] arp: Don't mix incoming and outgoing buffers
Date: Wed,  2 Apr 2025 19:23:26 +0200	[thread overview]
Message-ID: <20250402172343.858187-2-lvivier@redhat.com> (raw)
In-Reply-To: <20250402172343.858187-1-lvivier@redhat.com>

Don't use the memory of the incoming packet to build the outgoing buffer
as it can be memory of the TX queue in the case of vhost-user.

Moreover with vhost-user, the packet can be splitted accross several
iovec and it's easier to rebuild it in a buffer than updating an
existing iovec array.

Signed-off-by: Laurent Vivier <lvivier@redhat.com>
---
 arp.c | 84 ++++++++++++++++++++++++++++++++++++++---------------------
 1 file changed, 55 insertions(+), 29 deletions(-)

diff --git a/arp.c b/arp.c
index fc482bbd9938..9d68d7c3b602 100644
--- a/arp.c
+++ b/arp.c
@@ -31,56 +31,82 @@
 #include "tap.h"
 
 /**
  * arp() - Check if this is a supported ARP message, reply as needed
  * @c:		Execution context
- * @p:		Packet pool, single packet with Ethernet buffer
+ * @ah:		ARP header
+ * @am:		ARP message
  *
- * Return: 1 if handled, -1 on failure
+ * Return: true if the message is supported, false otherwise.
  */
-int arp(const struct ctx *c, const struct pool *p)
+static bool ignore_arp(const struct ctx *c,
+		       const struct arphdr *ah, const struct arpmsg *am)
 {
-	unsigned char swap[4];
-	struct ethhdr *eh;
-	struct arphdr *ah;
-	struct arpmsg *am;
-	size_t l2len;
-
-	eh = packet_get(p, 0, 0,			 sizeof(*eh), NULL);
-	ah = packet_get(p, 0, sizeof(*eh),		 sizeof(*ah), NULL);
-	am = packet_get(p, 0, sizeof(*eh) + sizeof(*ah), sizeof(*am), NULL);
-
-	if (!eh || !ah || !am)
-		return -1;
-
 	if (ah->ar_hrd != htons(ARPHRD_ETHER)	||
 	    ah->ar_pro != htons(ETH_P_IP)	||
 	    ah->ar_hln != ETH_ALEN		||
 	    ah->ar_pln != 4			||
 	    ah->ar_op  != htons(ARPOP_REQUEST))
-		return 1;
+		return true;
 
 	/* Discard announcements, but not 0.0.0.0 "probes" */
 	if (memcmp(am->sip, &in4addr_any, sizeof(am->sip)) &&
 	    !memcmp(am->sip, am->tip, sizeof(am->sip)))
-		return 1;
+		return true;
 
 	/* Don't resolve the guest's assigned address, either. */
 	if (!memcmp(am->tip, &c->ip4.addr, sizeof(am->tip)))
+		return true;
+
+	return false;
+}
+
+/**
+ * arp() - Check if this is a supported ARP message, reply as needed
+ * @c:		Execution context
+ * @p:		Packet pool, single packet with Ethernet buffer
+ *
+ * Return: 1 if handled, -1 on failure
+ */
+int arp(const struct ctx *c, const struct pool *p)
+{
+	struct {
+		struct ethhdr eh;
+		struct arphdr ah;
+		struct arpmsg am;
+	} __attribute__((__packed__)) resp;
+	const struct ethhdr *eh;
+	const struct arphdr *ah;
+	const struct arpmsg *am;
+
+	eh = packet_get(p, 0, 0,			 sizeof(*eh), NULL);
+	ah = packet_get(p, 0, sizeof(*eh),		 sizeof(*ah), NULL);
+	am = packet_get(p, 0, sizeof(*eh) + sizeof(*ah), sizeof(*am), NULL);
+
+	if (!eh || !ah || !am)
+		return -1;
+
+	if (ignore_arp(c, ah, am))
 		return 1;
 
-	ah->ar_op = htons(ARPOP_REPLY);
-	memcpy(am->tha,		am->sha,	sizeof(am->tha));
-	memcpy(am->sha,		c->our_tap_mac,	sizeof(am->sha));
+	/* ethernet header */
+	resp.eh.h_proto = htons(ETH_P_ARP);
+	memcpy(resp.eh.h_dest, eh->h_source, sizeof(resp.eh.h_dest));
+	memcpy(resp.eh.h_source, c->our_tap_mac, sizeof(resp.eh.h_source));
 
-	memcpy(swap,		am->tip,	sizeof(am->tip));
-	memcpy(am->tip,		am->sip,	sizeof(am->tip));
-	memcpy(am->sip,		swap,		sizeof(am->sip));
+	/* ARP header */
+	resp.ah.ar_op = htons(ARPOP_REPLY);
+	resp.ah.ar_hrd = ah->ar_hrd;
+	resp.ah.ar_pro = ah->ar_pro;
+	resp.ah.ar_hln = ah->ar_hln;
+	resp.ah.ar_pln = ah->ar_pln;
 
-	l2len = sizeof(*eh) + sizeof(*ah) + sizeof(*am);
-	memcpy(eh->h_dest,	eh->h_source,	sizeof(eh->h_dest));
-	memcpy(eh->h_source,	c->our_tap_mac,	sizeof(eh->h_source));
+	/* ARP message */
+	memcpy(resp.am.sha,		c->our_tap_mac,	sizeof(resp.am.sha));
+	memcpy(resp.am.sip,		am->tip,	sizeof(resp.am.sip));
+	memcpy(resp.am.tha,		am->sha,	sizeof(resp.am.tha));
+	memcpy(resp.am.tip,		am->sip,	sizeof(resp.am.tip));
 
-	tap_send_single(c, eh, l2len);
+	tap_send_single(c, &resp, sizeof(resp));
 
 	return 1;
 }
@@ -31,56 +31,82 @@
 
 /**
- * arp() - Check if this is a supported ARP message, reply as needed
+ * ignore_arp() - Check if this is a supported ARP message
  * @c:		Execution context
-- 
@@ -31,56 +31,82 @@
 #include "tap.h"
 
 /**
- * arp() - Check if this is a supported ARP message, reply as needed
+ * ignore_arp() - Check if this is a supported ARP message
  * @c:		Execution context
- * @p:		Packet pool, single packet with Ethernet buffer
+ * @ah:		ARP header
+ * @am:		ARP message
  *
- * Return: 1 if handled, -1 on failure
+ * Return: true if the message is supported, false otherwise.
  */
-int arp(const struct ctx *c, const struct pool *p)
+static bool ignore_arp(const struct ctx *c,
+		       const struct arphdr *ah, const struct arpmsg *am)
 {
-	unsigned char swap[4];
-	struct ethhdr *eh;
-	struct arphdr *ah;
-	struct arpmsg *am;
-	size_t l2len;
-
-	eh = packet_get(p, 0, 0,			 sizeof(*eh), NULL);
-	ah = packet_get(p, 0, sizeof(*eh),		 sizeof(*ah), NULL);
-	am = packet_get(p, 0, sizeof(*eh) + sizeof(*ah), sizeof(*am), NULL);
-
-	if (!eh || !ah || !am)
-		return -1;
-
 	if (ah->ar_hrd != htons(ARPHRD_ETHER)	||
 	    ah->ar_pro != htons(ETH_P_IP)	||
 	    ah->ar_hln != ETH_ALEN		||
 	    ah->ar_pln != 4			||
 	    ah->ar_op  != htons(ARPOP_REQUEST))
-		return 1;
+		return true;
 
 	/* Discard announcements, but not 0.0.0.0 "probes" */
 	if (memcmp(am->sip, &in4addr_any, sizeof(am->sip)) &&
 	    !memcmp(am->sip, am->tip, sizeof(am->sip)))
-		return 1;
+		return true;
 
 	/* Don't resolve the guest's assigned address, either. */
 	if (!memcmp(am->tip, &c->ip4.addr, sizeof(am->tip)))
+		return true;
+
+	return false;
+}
+
+/**
+ * arp() - Check if this is a supported ARP message, reply as needed
+ * @c:		Execution context
+ * @p:		Packet pool, single packet with Ethernet buffer
+ *
+ * Return: 1 if handled, -1 on failure
+ */
+int arp(const struct ctx *c, const struct pool *p)
+{
+	struct {
+		struct ethhdr eh;
+		struct arphdr ah;
+		struct arpmsg am;
+	} __attribute__((__packed__)) resp;
+	const struct ethhdr *eh;
+	const struct arphdr *ah;
+	const struct arpmsg *am;
+
+	eh = packet_get(p, 0, 0,			 sizeof(*eh), NULL);
+	ah = packet_get(p, 0, sizeof(*eh),		 sizeof(*ah), NULL);
+	am = packet_get(p, 0, sizeof(*eh) + sizeof(*ah), sizeof(*am), NULL);
+
+	if (!eh || !ah || !am)
+		return -1;
+
+	if (ignore_arp(c, ah, am))
 		return 1;
 
-	ah->ar_op = htons(ARPOP_REPLY);
-	memcpy(am->tha,		am->sha,	sizeof(am->tha));
-	memcpy(am->sha,		c->our_tap_mac,	sizeof(am->sha));
+	/* ethernet header */
+	resp.eh.h_proto = htons(ETH_P_ARP);
+	memcpy(resp.eh.h_dest, eh->h_source, sizeof(resp.eh.h_dest));
+	memcpy(resp.eh.h_source, c->our_tap_mac, sizeof(resp.eh.h_source));
 
-	memcpy(swap,		am->tip,	sizeof(am->tip));
-	memcpy(am->tip,		am->sip,	sizeof(am->tip));
-	memcpy(am->sip,		swap,		sizeof(am->sip));
+	/* ARP header */
+	resp.ah.ar_op = htons(ARPOP_REPLY);
+	resp.ah.ar_hrd = ah->ar_hrd;
+	resp.ah.ar_pro = ah->ar_pro;
+	resp.ah.ar_hln = ah->ar_hln;
+	resp.ah.ar_pln = ah->ar_pln;
 
-	l2len = sizeof(*eh) + sizeof(*ah) + sizeof(*am);
-	memcpy(eh->h_dest,	eh->h_source,	sizeof(eh->h_dest));
-	memcpy(eh->h_source,	c->our_tap_mac,	sizeof(eh->h_source));
+	/* ARP message */
+	memcpy(resp.am.sha,		c->our_tap_mac,	sizeof(resp.am.sha));
+	memcpy(resp.am.sip,		am->tip,	sizeof(resp.am.sip));
+	memcpy(resp.am.tha,		am->sha,	sizeof(resp.am.tha));
+	memcpy(resp.am.tip,		am->sip,	sizeof(resp.am.tip));
 
-	tap_send_single(c, eh, l2len);
+	tap_send_single(c, &resp, sizeof(resp));
 
 	return 1;
 }
-- 
2.49.0


  reply	other threads:[~2025-04-02 17:23 UTC|newest]

Thread overview: 37+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2025-04-02 17:23 [PATCH 00/18] Introduce discontiguous frames management Laurent Vivier
2025-04-02 17:23 ` Laurent Vivier [this message]
2025-04-03  1:57   ` [PATCH 01/18] arp: Don't mix incoming and outgoing buffers David Gibson
2025-04-02 17:23 ` [PATCH 02/18] iov: Update IOV_REMOVE_HEADER() and IOV_PEEK_HEADER() Laurent Vivier
2025-04-03  2:36   ` David Gibson
2025-04-02 17:23 ` [PATCH 03/18] tap: Use iov_tail with tap_add_packet() Laurent Vivier
2025-04-03  4:50   ` David Gibson
2025-04-02 17:23 ` [PATCH 04/18] packet: Use iov_tail with packet_add() Laurent Vivier
2025-04-03  4:54   ` David Gibson
2025-04-02 17:23 ` [PATCH 05/18] packet: Add packet_base() Laurent Vivier
2025-04-03  4:59   ` David Gibson
2025-04-02 17:23 ` [PATCH 06/18] arp: Convert to iov_tail Laurent Vivier
2025-04-03  5:00   ` David Gibson
2025-04-02 17:23 ` [PATCH 07/18] ndp: " Laurent Vivier
2025-04-03  5:01   ` David Gibson
2025-04-02 17:23 ` [PATCH 08/18] icmp: " Laurent Vivier
2025-04-03  5:04   ` David Gibson
2025-04-02 17:23 ` [PATCH 09/18] udp: " Laurent Vivier
2025-04-03  5:11   ` David Gibson
2025-04-02 17:23 ` [PATCH 10/18] tcp: Convert tcp_tap_handler() to use iov_tail Laurent Vivier
2025-04-03  5:14   ` David Gibson
2025-04-02 17:23 ` [PATCH 11/18] tcp: Convert tcp_data_from_tap() " Laurent Vivier
2025-04-03  5:20   ` David Gibson
2025-04-02 17:23 ` [PATCH 12/18] dhcpv6: Convert to iov_tail Laurent Vivier
2025-04-03  5:21   ` David Gibson
2025-04-02 17:23 ` [PATCH 13/18] dhcpv6: move offset initialization out of dhcpv6_opt() Laurent Vivier
2025-04-03  5:25   ` David Gibson
2025-04-02 17:23 ` [PATCH 14/18] dhcpv6: Use iov_tail in dhcpv6_opt() Laurent Vivier
2025-04-03  5:37   ` David Gibson
2025-04-02 17:23 ` [PATCH 15/18] dhcp: Convert to iov_tail Laurent Vivier
2025-04-03  5:47   ` David Gibson
2025-04-02 17:23 ` [PATCH 16/18] tap: " Laurent Vivier
2025-04-03 23:19   ` David Gibson
2025-04-02 17:23 ` [PATCH 17/18] ip: Use iov_tail in ipv6_l4hdr() Laurent Vivier
2025-04-03 23:21   ` David Gibson
2025-04-02 17:23 ` [PATCH 18/18] tap: Convert to iov_tail Laurent Vivier
2025-04-03 23:26   ` David Gibson

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20250402172343.858187-2-lvivier@redhat.com \
    --to=lvivier@redhat.com \
    --cc=passt-dev@passt.top \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
Code repositories for project(s) associated with this public inbox

	https://passt.top/passt

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for IMAP folder(s).